<!DOCTYPE kpartplugin>
<kpartplugin library="kget_browser_integration" name="kget_browser_integration" version="4">
<MenuBar>
    <Menu name="tools"><text>&amp;Tools</text>
        <Action name="kget_menu"/>
    </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
    <Action name="kget_menu"/>
</ToolBar>
</kpartplugin>
