<!DOCTYPE kpartgui>
<kpartgui name="kget" version="8">
<MenuBar>
    <Menu noMerge="1" name="file"><text>&amp;File</text>
        <Action name="new_download"/>
        <Action name="import_links"/>
        <Separator/>
        <Action name="import_transfers"/>
        <Action name="export_transfers"/>
        <Action name="create_metalink"/>
        <Separator/>
        <Action name="transfer_history" />
        <Separator/>
        <Action name="file_quit"/>
    </Menu>
        <Menu noMerge="1" name="Downloads"><text>&amp;Downloads</text>
        <Action name="start_all_download"/>
        <Action name="start_selected_download"/>
        <Separator/>        
        <Action name="stop_all_download"/>
        <Action name="stop_selected_download"/>
        <Separator/>
        <Action name="priority_top"/>
        <Action name="priority_up"/>
        <Action name="priority_down"/>
        <Action name="priority_bottom"/>
        <Separator/>
        <Action name="delete_selected_download"/>
        <Action name="delete_all_finished"/>
        <Separator/>
        <Action name="edit_groups"/>
    </Menu>
   <Menu name="settings"><text>&amp;Settings</text>
        <Action name="show_drop_target"/>
    </Menu>
    <Menu name="help"><text>&amp;Help</text>
    </Menu>
</MenuBar>
<ToolBar fullWidth="true" name="kget_toolbar"><text>Main Toolbar</text>
    <Action name="new_download"/>
    <Separator lineSeparator="true"/>
    <Action name="start_menu"/>
    <Action name="stop_menu"/>
    <Action name="delete_selected_download"/>
    <Action name="delete_all_finished"/>
</ToolBar>
</kpartgui>
