/*
    This file is part of the KDE project
    Copyright (C) 2011 Ernesto Rodriguez Ortiz <eortiz@uci.cu>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef DLGMMSTHREAD_H
#define DLGMMSTHREAD_H
#include <KCModule>

#include "ui_dlgmms.h"

class DlgMmsSettings : public KCModule
{
    Q_OBJECT
    public:
        explicit DlgMmsSettings(QWidget * parent = 0, const QVariantList &args = QVariantList());

    public slots:
        virtual void save();
        virtual void load();

    private:
        Ui::DlgMms ui;
};

#endif // DLGMMSTHREAD_H
