/* This file is part of the KDE project

   Copyright (C) 2007 Urs Wolfer <uwolfer @ kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
*/

#include "core/transfercontainer.h"

#include <KDebug>

TransferContainer::TransferContainer()
{
}

TransferContainer::~TransferContainer()
{
}

#include "transfercontainer.moc"
